package storageaccounts

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See NOTICE.txt in the project root for license information.

type StorageAccountListKeysResult struct {
	Keys *[]StorageAccountKey `json:"keys,omitempty"`
}
