// Code generated by smithy-go-codegen DO NOT EDIT.

// Package signin provides the API client, operations, and parameter types for AWS
// Sign-In Service.
//
// AWS Sign-In manages authentication for AWS services. This service provides
// secure authentication flows for accessing AWS resources from the console and
// developer tools.
package signin
