// Code generated by smithy-go-codegen DO NOT EDIT.

// Package iam provides the API client, operations, and parameter types for AWS
// Identity and Access Management.
//
// # Identity and Access Management
//
// Identity and Access Management (IAM) is a web service for securely controlling
// access to Amazon Web Services services. With IAM, you can centrally manage
// users, security credentials such as access keys, and permissions that control
// which Amazon Web Services resources users and applications can access. For more
// information about IAM, see [Identity and Access Management (IAM)]and the [Identity and Access Management User Guide].
//
// [Identity and Access Management User Guide]: https://docs.aws.amazon.com/IAM/latest/UserGuide/
// [Identity and Access Management (IAM)]: http://aws.amazon.com/iam/
package iam
