// Code generated by MockGen. DO NOT EDIT.
// Source: admin_setting_customization.go
//
// Generated by this command:
//
//	mockgen -source=admin_setting_customization.go -destination=mocks/admin_setting_customization_mocks.go -package=mocks
//

// Package mocks is a generated GoMock package.
package mocks

import (
	context "context"
	reflect "reflect"

	tfe "github.com/hashicorp/go-tfe"
	gomock "go.uber.org/mock/gomock"
)

// MockCustomizationSettings is a mock of CustomizationSettings interface.
type MockCustomizationSettings struct {
	ctrl     *gomock.Controller
	recorder *MockCustomizationSettingsMockRecorder
}

// MockCustomizationSettingsMockRecorder is the mock recorder for MockCustomizationSettings.
type MockCustomizationSettingsMockRecorder struct {
	mock *MockCustomizationSettings
}

// NewMockCustomizationSettings creates a new mock instance.
func NewMockCustomizationSettings(ctrl *gomock.Controller) *MockCustomizationSettings {
	mock := &MockCustomizationSettings{ctrl: ctrl}
	mock.recorder = &MockCustomizationSettingsMockRecorder{mock}
	return mock
}

// EXPECT returns an object that allows the caller to indicate expected use.
func (m *MockCustomizationSettings) EXPECT() *MockCustomizationSettingsMockRecorder {
	return m.recorder
}

// Read mocks base method.
func (m *MockCustomizationSettings) Read(ctx context.Context) (*tfe.AdminCustomizationSetting, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Read", ctx)
	ret0, _ := ret[0].(*tfe.AdminCustomizationSetting)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// Read indicates an expected call of Read.
func (mr *MockCustomizationSettingsMockRecorder) Read(ctx any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Read", reflect.TypeOf((*MockCustomizationSettings)(nil).Read), ctx)
}

// Update mocks base method.
func (m *MockCustomizationSettings) Update(ctx context.Context, options tfe.AdminCustomizationSettingsUpdateOptions) (*tfe.AdminCustomizationSetting, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Update", ctx, options)
	ret0, _ := ret[0].(*tfe.AdminCustomizationSetting)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// Update indicates an expected call of Update.
func (mr *MockCustomizationSettingsMockRecorder) Update(ctx, options any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Update", reflect.TypeOf((*MockCustomizationSettings)(nil).Update), ctx, options)
}
