package storageaccounts

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See NOTICE.txt in the project root for license information.

type RoutingPreference struct {
	PublishInternetEndpoints  *bool          `json:"publishInternetEndpoints,omitempty"`
	PublishMicrosoftEndpoints *bool          `json:"publishMicrosoftEndpoints,omitempty"`
	RoutingChoice             *RoutingChoice `json:"routingChoice,omitempty"`
}
