// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

// Package internal provides internal utilities for the OpenTelemetry prometheus exporter.
package internal // import "go.opentelemetry.io/otel/exporters/prometheus/internal"

// Version is the current release version of the OpenTelemetry prometheus
// exporter in use.
const Version = "0.65.0"
