// Copyright (c) HashiCorp, Inc.
// SPDX-License-Identifier: MPL-2.0

package environments

const analysisServicesAppId = "4ac7d521-0382-477b-b0f8-7e1d95f85ca2"
const apiManagementAppId = "8602e328-9b72-4f2d-a4ae-1387d013a2b3"
const appConfigurationAppId = "35ffadb3-7fc1-497e-b61b-381d28e744cc"
const appServiceAppId = "abfa0a7c-a6b6-4736-8310-5855508787cd"
const applicationInsightsAppId = "f5c26e74-f226-4ae8-85f0-b4af0080ac9e"
const attestationServiceAppId = "c61423b7-1d1f-430d-b444-0eee53298103"
const azureAdIdentityGovernanceInsightsAppId = "58c746b0-a0b0-4647-a8f6-12dde5981638"
const azureAdIntegratedAppAppId = "af47b99c-8954-4b45-ab68-8121157418ef"
const azureAdNotificationAppId = "fc03f97a-9db0-4627-a216-ec98ce54e018"
const azureDevOpsAppId = "499b84ac-1321-427f-aa17-267ca6975798"
const azureLinuxVmSignInAppId = "ce6ff14a-7fdc-4685-bbe0-f6afdfcfa8e0"
const azureWindowsVmSignInAppId = "372140e0-b3b7-4226-8ef9-d57986796201"
const azureServiceManagementAppId = "797f4846-ba00-4fd7-ba43-dac1f8f63013"
const azureVPNAppId = "41b23e61-6c1e-4545-b367-cd054e0ed4b4"
const batchAppId = "ddbf3205-c6bd-46ae-8127-60eb93363864"
const bingAppId = "9ea1ad79-fdb6-4f9a-8bc3-2b70f96e34c7"
const botFrameworkDevPortalAppId = "f3723d34-6ff5-4ceb-a148-d99dcd2511fc"
const branchConnectWebServiceAppId = "57084ef3-d413-4087-a28f-f6f3b1ad7786"
const cdnFrontDoorAppId = "205478c0-bd83-4e1b-a9d6-db63a3e1e1c8"
const cognitiveServicesAppId = "7d312290-28c8-473c-a0ed-8e53749b6d6d"
const computeRecommendationServiceAppId = "b9a92e36-2cf8-4f4e-bcb3-9d99e00e14ab"
const connectionsServiceAppId = "b7912db9-aa33-4820-9d4f-709830fdd78f"
const containerRegistryAppId = "6a0ec4d3-30cb-4a83-91c0-ae56bc0e3d26"
const cortanaAtWorkBingServicesAppId = "22d7579f-06c2-4baa-89d2-e844486adb9d"
const cortanaAtWorkServiceAppId = "2a486b53-dbd2-49c0-a2bc-278bdfc30833"
const cortanaRuntimeServiceAppId = "81473081-50b9-469a-b9d8-303109583ecb"
const cosmosDBAppId = "a232010e-820c-4083-83bb-3ace5fc29d0b"
const customerInsightsAppId = "38c77d00-5fcb-4cce-9d93-af4738258e3c"
const dataBricksAppId = "2ff814a6-3304-4ab8-85cb-cd0e6f879c1d"
const dataCatalogAppId = "9d3e55ba-79e0-4b7c-af50-dc460b81dca1"
const dataLakeAppId = "e9f49c6b-5ce5-44c8-925d-015017e9f7ad"
const dataMigrationsServiceAppId = "a4bad4aa-bf02-4631-9f78-a64ffdba8150"
const digitalTwinsAppId = "0b07f429-9f4b-4714-9392-cc5e8e80c8b0"
const domainControllerServiceAppId = "2565bd9d-da50-47d4-8b85-4c97f669dc36"
const dynamic365BusinessCentralAppId = "996def3d-b36c-4153-8607-a6fd3c01b89f"
const dynamics365DataExportServiceAppId = "b861dbcc-a7ef-4219-a005-0e4de4ea7dcf"
const dynamicsCRMAppId = "00000007-0000-0000-c000-000000000000"
const dynamicsERPAppId = "00000015-0000-0000-c000-000000000000"
const eventHubsAppId = "80369ed6-5f11-4dd9-bef3-692475845e77"
const flowServiceAppId = "7df0a125-d3be-4c96-aa54-591f83ff541c"
const graphConnectorServiceAppId = "56c1da01-2129-48f7-9355-af6d59d42766"
const hdinsightClusterAppId = "7865c1d2-f040-46cc-875f-831a1ef6a28a"
const healthCareAppId = "4f6778d8-5aef-43dc-a1ff-b073724b9495"
const iamSupportabilityAppId = "a57aca87-cbc0-4f3c-8b9e-dc095fdc8978"
const importExportAppId = "7de4d5c5-5b32-4235-b8a9-33b34d6bcd2a"
const inTuneAppId = "c161e42e-d4df-4a3d-9b42-e7a3c31f59d4"
const informationProtectionSyncServiceAppId = "870c4f2e-85b6-4d43-bdda-6ed9a579b725"
const iotCentralAppId = "9edfcdd9-0bc5-4bd4-b287-c3afc716aac7"
const iotHubDeviceProvisioningAppId = "0cd79364-7a90-4354-9984-6e36c841418d"
const keyVaultAppId = "cfa8b339-82a2-471a-a3c9-0fc0be7a4093"
const kubernetesServiceAadServerAppId = "6dae42f8-4368-4678-94ff-3960e28e3630"
const kustoServiceAppId = "2746ea77-4702-4b45-80ca-3c97e680e8b7"
const kustoServiceMFAAppId = "725d0e77-e1fd-48f1-a295-2115457f7609"
const logAnalyticsAppId = "ca7f3f0b-7d91-482c-8e09-c5d840d0eac5"
const managedHSMAppId = "1341df96-0b28-43da-ba24-7a6ce39be816"
const mapsAppId = "ba1ea022-5807-41d5-bbeb-292c7e1cf5f6"
const mediaServicesAppId = "374b2a64-3b6b-436b-934c-b820eacca870"
const microsoft365DataAtRestEncryptionAppId = "c066d759-24ae-40e7-a56f-027002b5d3e4"
const microsoftAppAccessPanelAppId = "0000000c-0000-0000-c000-000000000000"
const microsoftAzureCliAppId = "04b07795-8ddb-461a-bbee-02f9e1bf7b46"
const microsoftGraphAppId = "00000003-0000-0000-c000-000000000000"
const microsoftInvoicingAppId = "b6b84568-6c01-4981-a80f-09da9a20bbed"
const microsoftIntuneAppId = "0000000a-0000-0000-c000-000000000000"
const microsoftIntuneEnrollmentAppId = "d4ebce55-015a-49b5-a083-c84d1797ae8c"
const microsoftOfficeAppId = "d3590ed6-52b3-4102-aeff-aad2292ab01c"
const microsoftStorageSyncAppId = "9469b9f5-6722-4481-a2b2-14ed560b706f"
const microsoftTeamsAppId = "1fec8e78-bce4-4aaf-ab1b-5451cc387264"
const microsoftTeamsWebClientAppId = "5e3ce6c0-2b1f-4285-8d4b-75ee78787346"
const mileIqAdminCenterAppId = "de096ee1-dae7-4ee1-8dd5-d88ccc473815"
const mileIqDashboardAppId = "f7069a8d-9edc-4300-b365-ae53c9627fc4"
const mileIqRestServiceAppId = "b692184e-b47f-4706-b352-84b288d2d9ee"
const mixedRealityAppId = "c7ddd9b4-5172-4e28-bd29-1e0792947d18"
const office365ConnectorsAppId = "48af08dc-f6d2-435f-b2a7-069abd99c086"
const office365DemeterAppId = "982bda36-4632-4165-a46a-9863b1bbcf7d"
const office365DwEngineV2AppId = "441509e5-a165-4363-8ee7-bcf0b7d26739"
const office365ExchangeOnlineAppId = "00000002-0000-0ff1-ce00-000000000000"
const office365ExchangeOnlineProtectionAppId = "00000007-0000-0ff1-ce00-000000000000"
const office365InformationProtectionAppId = "2f3f02c9-5679-4a5c-a605-0de55b07d135"
const office365ManagementAppId = "c5393580-f805-4401-95e8-94b7a6ef2fc2"
const office365PortalAppId = "00000006-0000-0ff1-ce00-000000000000"
const office365SharePointOnlineAppId = "00000003-0000-0ff1-ce00-000000000000"
const office365SuiteUxAppId = "4345a7b9-9a63-4910-a426-35363201d503"
const office365ZoomAppId = "0d38933a-0bbd-41ca-9ebd-28c4b5ba7cb7"
const officeHomeAppId = "4765445b-32c6-49b0-83e6-1d93765276ca"
const officeUwpPwaAppId = "0ec893e0-5785-4de6-99da-4ed124e5296c"
const oneNoteAppId = "2d4d3d8e-2be3-4bef-9f87-7875a61c29de"
const oneProfileServiceAppId = "b2cc270f-563e-4d8a-af47-f00963a71dcd"
const ossRDBMSAppId = "123cd850-d9df-40bd-94d5-c9f07b7fa203"
const ossRdbmsPostgreSqlFlexibleServerAadAuthenticationAppId = "5657e26c-cc92-45d9-bc47-9da6cfdb4ed9"
const peopleCardsServiceAppId = "394866fc-eedb-4f01-8536-3ff84b16be2a"
const policyAdministrationServiceAppId = "0469d4cd-df37-4d93-8a61-f8c75b809164"
const portalAppId = "c44b4083-3bb0-49c1-b47d-974e53cbdf3c"
const powerAppsRuntimeServiceAppId = "82f77645-8a66-4745-bcdf-9706824f9ad0"
const powerBiServiceAppId = "00000009-0000-0000-c000-000000000000"
const purviewAppId = "73c2949e-da2d-457a-9607-fcc665198967"
const rightsManagementServiceAppId = "00000012-0000-0000-c000-000000000000"
const securityInsightsAppId = "98785600-1bb7-4fb9-b9fa-19afe2c8a360"
const serviceBusAppId = "80a10ef9-8168-493d-abf9-3297c4ef6e3c"
const serviceDeployAppId = "5b306cba-9c71-49db-96c3-d17ca2379c4d"
const serviceTrustAppId = "d6fdaa33-e821-4211-83d0-cf74736489e1"
const signupAppId = "b4bddae8-ab25-483e-8670-df09b9f1d0ea"
const skypeForBusinessOnlineAppId = "00000004-0000-0ff1-ce00-000000000000"
const speechRecognitionAppId = "1a6fcee6-0816-469b-acac-fe7ef2e87b83"
const sqlDatabaseAppId = "022907d3-0f1b-48f7-badc-1ba6abab6d66"
const stackHciServiceAppId = "1322e676-dee7-41ee-a874-ac923822781c"
const storageAppId = "e406a681-f3d4-42a8-90b6-c2b029497af1"
const storageSyncAppId = "9469b9f5-6722-4481-a2b2-14ed560b706f"
const streamAnalyticsAppId = "66f1e791-7bfb-4e18-aed8-1720056421c7"
const synapseAppId = "9e09aefc-b2e5-4d19-9f74-3e3e8b11a57b"
const synapseGatewayAppId = "1ac05c7e-12d2-4605-bf9d-549d7041c6b3"
const synapseStudioAppId = "ec52d13d-2e85-410e-a89a-8c79fb6a32ac"
const targetedMessagingServiceAppId = "4c4f550b-42b2-4a16-93f9-fdb9e01bb6ed"
const teamsServiceAppId = "cc15fd57-2c6c-4117-a88c-83b1d56b4bbe"
const threatProtectionAppId = "8ee8fdad-f234-4243-8f3b-15c294843740"
const timeSeriesInsightsAppId = "120d688d-1518-4cf7-bd38-182f158850b6"
const trafficManagerAppId = "2cf9eb86-36b5-49dc-86ae-9a63135dfa8c"
const universalPrintAppId = "da9b70f6-5323-4ce6-ae5c-88dcc5082966"
const windowsDefenderAtpAppId = "fc780465-2017-40d4-a0c5-307022471b92"
const windowsVirtualDesktopAppId = "9cdead84-a844-4324-93f2-b2e6bb768d07"
const yammerAppId = "00000005-0000-0ff1-ce00-000000000000"
