// Code generated by smithy-go-codegen DO NOT EDIT.

package types

type MessageSystemAttributeName string

// Enum values for MessageSystemAttributeName
const (
	MessageSystemAttributeNameAll                              MessageSystemAttributeName = "All"
	MessageSystemAttributeNameSenderId                         MessageSystemAttributeName = "SenderId"
	MessageSystemAttributeNameSentTimestamp                    MessageSystemAttributeName = "SentTimestamp"
	MessageSystemAttributeNameApproximateReceiveCount          MessageSystemAttributeName = "ApproximateReceiveCount"
	MessageSystemAttributeNameApproximateFirstReceiveTimestamp MessageSystemAttributeName = "ApproximateFirstReceiveTimestamp"
	MessageSystemAttributeNameSequenceNumber                   MessageSystemAttributeName = "SequenceNumber"
	MessageSystemAttributeNameMessageDeduplicationId           MessageSystemAttributeName = "MessageDeduplicationId"
	MessageSystemAttributeNameMessageGroupId                   MessageSystemAttributeName = "MessageGroupId"
	MessageSystemAttributeNameAWSTraceHeader                   MessageSystemAttributeName = "AWSTraceHeader"
	MessageSystemAttributeNameDeadLetterQueueSourceArn         MessageSystemAttributeName = "DeadLetterQueueSourceArn"
)

// Values returns all known values for MessageSystemAttributeName. Note that this
// can be expanded in the future, and so it is only as up to date as the client.
//
// The ordering of this slice is not guaranteed to be stable across updates.
func (MessageSystemAttributeName) Values() []MessageSystemAttributeName {
	return []MessageSystemAttributeName{
		"All",
		"SenderId",
		"SentTimestamp",
		"ApproximateReceiveCount",
		"ApproximateFirstReceiveTimestamp",
		"SequenceNumber",
		"MessageDeduplicationId",
		"MessageGroupId",
		"AWSTraceHeader",
		"DeadLetterQueueSourceArn",
	}
}

type MessageSystemAttributeNameForSends string

// Enum values for MessageSystemAttributeNameForSends
const (
	MessageSystemAttributeNameForSendsAWSTraceHeader MessageSystemAttributeNameForSends = "AWSTraceHeader"
)

// Values returns all known values for MessageSystemAttributeNameForSends. Note
// that this can be expanded in the future, and so it is only as up to date as the
// client.
//
// The ordering of this slice is not guaranteed to be stable across updates.
func (MessageSystemAttributeNameForSends) Values() []MessageSystemAttributeNameForSends {
	return []MessageSystemAttributeNameForSends{
		"AWSTraceHeader",
	}
}

type QueueAttributeName string

// Enum values for QueueAttributeName
const (
	QueueAttributeNameAll                                   QueueAttributeName = "All"
	QueueAttributeNamePolicy                                QueueAttributeName = "Policy"
	QueueAttributeNameVisibilityTimeout                     QueueAttributeName = "VisibilityTimeout"
	QueueAttributeNameMaximumMessageSize                    QueueAttributeName = "MaximumMessageSize"
	QueueAttributeNameMessageRetentionPeriod                QueueAttributeName = "MessageRetentionPeriod"
	QueueAttributeNameApproximateNumberOfMessages           QueueAttributeName = "ApproximateNumberOfMessages"
	QueueAttributeNameApproximateNumberOfMessagesNotVisible QueueAttributeName = "ApproximateNumberOfMessagesNotVisible"
	QueueAttributeNameCreatedTimestamp                      QueueAttributeName = "CreatedTimestamp"
	QueueAttributeNameLastModifiedTimestamp                 QueueAttributeName = "LastModifiedTimestamp"
	QueueAttributeNameQueueArn                              QueueAttributeName = "QueueArn"
	QueueAttributeNameApproximateNumberOfMessagesDelayed    QueueAttributeName = "ApproximateNumberOfMessagesDelayed"
	QueueAttributeNameDelaySeconds                          QueueAttributeName = "DelaySeconds"
	QueueAttributeNameReceiveMessageWaitTimeSeconds         QueueAttributeName = "ReceiveMessageWaitTimeSeconds"
	QueueAttributeNameRedrivePolicy                         QueueAttributeName = "RedrivePolicy"
	QueueAttributeNameFifoQueue                             QueueAttributeName = "FifoQueue"
	QueueAttributeNameContentBasedDeduplication             QueueAttributeName = "ContentBasedDeduplication"
	QueueAttributeNameKmsMasterKeyId                        QueueAttributeName = "KmsMasterKeyId"
	QueueAttributeNameKmsDataKeyReusePeriodSeconds          QueueAttributeName = "KmsDataKeyReusePeriodSeconds"
	QueueAttributeNameDeduplicationScope                    QueueAttributeName = "DeduplicationScope"
	QueueAttributeNameFifoThroughputLimit                   QueueAttributeName = "FifoThroughputLimit"
	QueueAttributeNameRedriveAllowPolicy                    QueueAttributeName = "RedriveAllowPolicy"
	QueueAttributeNameSqsManagedSseEnabled                  QueueAttributeName = "SqsManagedSseEnabled"
)

// Values returns all known values for QueueAttributeName. Note that this can be
// expanded in the future, and so it is only as up to date as the client.
//
// The ordering of this slice is not guaranteed to be stable across updates.
func (QueueAttributeName) Values() []QueueAttributeName {
	return []QueueAttributeName{
		"All",
		"Policy",
		"VisibilityTimeout",
		"MaximumMessageSize",
		"MessageRetentionPeriod",
		"ApproximateNumberOfMessages",
		"ApproximateNumberOfMessagesNotVisible",
		"CreatedTimestamp",
		"LastModifiedTimestamp",
		"QueueArn",
		"ApproximateNumberOfMessagesDelayed",
		"DelaySeconds",
		"ReceiveMessageWaitTimeSeconds",
		"RedrivePolicy",
		"FifoQueue",
		"ContentBasedDeduplication",
		"KmsMasterKeyId",
		"KmsDataKeyReusePeriodSeconds",
		"DeduplicationScope",
		"FifoThroughputLimit",
		"RedriveAllowPolicy",
		"SqsManagedSseEnabled",
	}
}
