// Code generated by smithy-go-codegen DO NOT EDIT.

// Package sns provides the API client, operations, and parameter types for Amazon
// Simple Notification Service.
//
// # Amazon Simple Notification Service
//
// Amazon Simple Notification Service (Amazon SNS) is a web service that enables
// you to build distributed web-enabled applications. Applications can use Amazon
// SNS to easily push real-time notification messages to interested subscribers
// over multiple delivery protocols. For more information about this product see
// the [Amazon SNS product page]. For detailed information about Amazon SNS features and their associated
// API calls, see the [Amazon SNS Developer Guide].
//
// For information on the permissions you need to use this API, see [Identity and access management in Amazon SNS] in the Amazon
// SNS Developer Guide.
//
// We also provide SDKs that enable you to access Amazon SNS from your preferred
// programming language. The SDKs contain functionality that automatically takes
// care of tasks such as: cryptographically signing your service requests, retrying
// requests, and handling error responses. For a list of available SDKs, go to [Tools for Amazon Web Services].
//
// [Amazon SNS product page]: http://aws.amazon.com/sns/
// [Identity and access management in Amazon SNS]: https://docs.aws.amazon.com/sns/latest/dg/sns-authentication-and-access-control.html
// [Tools for Amazon Web Services]: http://aws.amazon.com/tools/
// [Amazon SNS Developer Guide]: https://docs.aws.amazon.com/sns/latest/dg/
package sns
