// Code generated by "stringer -type ConstraintDepth"; DO NOT EDIT.

package constraints

import "strconv"

const _ConstraintDepth_name = "UnconstrainedConstrainedMajorConstrainedMinorConstrainedPatch"

var _ConstraintDepth_index = [...]uint8{0, 13, 29, 45, 61}

func (i ConstraintDepth) String() string {
	if i < 0 || i >= ConstraintDepth(len(_ConstraintDepth_index)-1) {
		return "ConstraintDepth(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _ConstraintDepth_name[_ConstraintDepth_index[i]:_ConstraintDepth_index[i+1]]
}
