// Code generated by "stringer -type=instanceType instancetype.go"; DO NOT EDIT.

package terraform

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[typeInvalid-0]
	_ = x[typePrimary-1]
	_ = x[typeTainted-2]
	_ = x[typeDeposed-3]
}

const _instanceType_name = "typeInvalidtypePrimarytypeTaintedtypeDeposed"

var _instanceType_index = [...]uint8{0, 11, 22, 33, 44}

func (i instanceType) String() string {
	if i < 0 || i >= instanceType(len(_instanceType_index)-1) {
		return "instanceType(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _instanceType_name[_instanceType_index[i]:_instanceType_index[i+1]]
}
