package storageaccounts

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See NOTICE.txt in the project root for license information.

type EncryptionIdentity struct {
	FederatedIdentityClientId *string `json:"federatedIdentityClientId,omitempty"`
	UserAssignedIdentity      *string `json:"userAssignedIdentity,omitempty"`
}
