// Copyright The OpenTelemetry Authors
// SPDX-License-Identifier: Apache-2.0

// Package autoexport provides OpenTelemetry exporter factory functions
// with defaults and environment variable support as defined by the
// OpenTelemetry specification.
package autoexport // import "go.opentelemetry.io/contrib/exporters/autoexport"
