package storageaccounts

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See NOTICE.txt in the project root for license information.

type StorageAccountCheckNameAvailabilityParameters struct {
	Name string `json:"name"`
	Type Type   `json:"type"`
}
