// Code generated by MockGen. DO NOT EDIT.
// Source: user.go
//
// Generated by this command:
//
//	mockgen -source=user.go -destination=mocks/user_mocks.go -package=mocks
//

// Package mocks is a generated GoMock package.
package mocks

import (
	context "context"
	reflect "reflect"

	tfe "github.com/hashicorp/go-tfe"
	gomock "go.uber.org/mock/gomock"
)

// MockUsers is a mock of Users interface.
type MockUsers struct {
	ctrl     *gomock.Controller
	recorder *MockUsersMockRecorder
}

// MockUsersMockRecorder is the mock recorder for MockUsers.
type MockUsersMockRecorder struct {
	mock *MockUsers
}

// NewMockUsers creates a new mock instance.
func NewMockUsers(ctrl *gomock.Controller) *MockUsers {
	mock := &MockUsers{ctrl: ctrl}
	mock.recorder = &MockUsersMockRecorder{mock}
	return mock
}

// EXPECT returns an object that allows the caller to indicate expected use.
func (m *MockUsers) EXPECT() *MockUsersMockRecorder {
	return m.recorder
}

// ReadCurrent mocks base method.
func (m *MockUsers) ReadCurrent(ctx context.Context) (*tfe.User, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "ReadCurrent", ctx)
	ret0, _ := ret[0].(*tfe.User)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// ReadCurrent indicates an expected call of ReadCurrent.
func (mr *MockUsersMockRecorder) ReadCurrent(ctx any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "ReadCurrent", reflect.TypeOf((*MockUsers)(nil).ReadCurrent), ctx)
}

// UpdateCurrent mocks base method.
func (m *MockUsers) UpdateCurrent(ctx context.Context, options tfe.UserUpdateOptions) (*tfe.User, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "UpdateCurrent", ctx, options)
	ret0, _ := ret[0].(*tfe.User)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// UpdateCurrent indicates an expected call of UpdateCurrent.
func (mr *MockUsersMockRecorder) UpdateCurrent(ctx, options any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "UpdateCurrent", reflect.TypeOf((*MockUsers)(nil).UpdateCurrent), ctx, options)
}
