// Code generated by MockGen. DO NOT EDIT.
// Source: registry_module.go
//
// Generated by this command:
//
//	mockgen -source=registry_module.go -destination=mocks/registry_module_mocks.go -package=mocks
//

// Package mocks is a generated GoMock package.
package mocks

import (
	context "context"
	io "io"
	reflect "reflect"

	tfe "github.com/hashicorp/go-tfe"
	gomock "go.uber.org/mock/gomock"
)

// MockRegistryModules is a mock of RegistryModules interface.
type MockRegistryModules struct {
	ctrl     *gomock.Controller
	recorder *MockRegistryModulesMockRecorder
}

// MockRegistryModulesMockRecorder is the mock recorder for MockRegistryModules.
type MockRegistryModulesMockRecorder struct {
	mock *MockRegistryModules
}

// NewMockRegistryModules creates a new mock instance.
func NewMockRegistryModules(ctrl *gomock.Controller) *MockRegistryModules {
	mock := &MockRegistryModules{ctrl: ctrl}
	mock.recorder = &MockRegistryModulesMockRecorder{mock}
	return mock
}

// EXPECT returns an object that allows the caller to indicate expected use.
func (m *MockRegistryModules) EXPECT() *MockRegistryModulesMockRecorder {
	return m.recorder
}

// Create mocks base method.
func (m *MockRegistryModules) Create(ctx context.Context, organization string, options tfe.RegistryModuleCreateOptions) (*tfe.RegistryModule, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Create", ctx, organization, options)
	ret0, _ := ret[0].(*tfe.RegistryModule)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// Create indicates an expected call of Create.
func (mr *MockRegistryModulesMockRecorder) Create(ctx, organization, options any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Create", reflect.TypeOf((*MockRegistryModules)(nil).Create), ctx, organization, options)
}

// CreateVersion mocks base method.
func (m *MockRegistryModules) CreateVersion(ctx context.Context, moduleID tfe.RegistryModuleID, options tfe.RegistryModuleCreateVersionOptions) (*tfe.RegistryModuleVersion, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "CreateVersion", ctx, moduleID, options)
	ret0, _ := ret[0].(*tfe.RegistryModuleVersion)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// CreateVersion indicates an expected call of CreateVersion.
func (mr *MockRegistryModulesMockRecorder) CreateVersion(ctx, moduleID, options any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "CreateVersion", reflect.TypeOf((*MockRegistryModules)(nil).CreateVersion), ctx, moduleID, options)
}

// CreateWithVCSConnection mocks base method.
func (m *MockRegistryModules) CreateWithVCSConnection(ctx context.Context, options tfe.RegistryModuleCreateWithVCSConnectionOptions) (*tfe.RegistryModule, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "CreateWithVCSConnection", ctx, options)
	ret0, _ := ret[0].(*tfe.RegistryModule)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// CreateWithVCSConnection indicates an expected call of CreateWithVCSConnection.
func (mr *MockRegistryModulesMockRecorder) CreateWithVCSConnection(ctx, options any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "CreateWithVCSConnection", reflect.TypeOf((*MockRegistryModules)(nil).CreateWithVCSConnection), ctx, options)
}

// Delete mocks base method.
func (m *MockRegistryModules) Delete(ctx context.Context, organization, name string) error {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Delete", ctx, organization, name)
	ret0, _ := ret[0].(error)
	return ret0
}

// Delete indicates an expected call of Delete.
func (mr *MockRegistryModulesMockRecorder) Delete(ctx, organization, name any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Delete", reflect.TypeOf((*MockRegistryModules)(nil).Delete), ctx, organization, name)
}

// DeleteByName mocks base method.
func (m *MockRegistryModules) DeleteByName(ctx context.Context, module tfe.RegistryModuleID) error {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "DeleteByName", ctx, module)
	ret0, _ := ret[0].(error)
	return ret0
}

// DeleteByName indicates an expected call of DeleteByName.
func (mr *MockRegistryModulesMockRecorder) DeleteByName(ctx, module any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "DeleteByName", reflect.TypeOf((*MockRegistryModules)(nil).DeleteByName), ctx, module)
}

// DeleteProvider mocks base method.
func (m *MockRegistryModules) DeleteProvider(ctx context.Context, moduleID tfe.RegistryModuleID) error {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "DeleteProvider", ctx, moduleID)
	ret0, _ := ret[0].(error)
	return ret0
}

// DeleteProvider indicates an expected call of DeleteProvider.
func (mr *MockRegistryModulesMockRecorder) DeleteProvider(ctx, moduleID any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "DeleteProvider", reflect.TypeOf((*MockRegistryModules)(nil).DeleteProvider), ctx, moduleID)
}

// DeleteVersion mocks base method.
func (m *MockRegistryModules) DeleteVersion(ctx context.Context, moduleID tfe.RegistryModuleID, version string) error {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "DeleteVersion", ctx, moduleID, version)
	ret0, _ := ret[0].(error)
	return ret0
}

// DeleteVersion indicates an expected call of DeleteVersion.
func (mr *MockRegistryModulesMockRecorder) DeleteVersion(ctx, moduleID, version any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "DeleteVersion", reflect.TypeOf((*MockRegistryModules)(nil).DeleteVersion), ctx, moduleID, version)
}

// List mocks base method.
func (m *MockRegistryModules) List(ctx context.Context, organization string, options *tfe.RegistryModuleListOptions) (*tfe.RegistryModuleList, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "List", ctx, organization, options)
	ret0, _ := ret[0].(*tfe.RegistryModuleList)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// List indicates an expected call of List.
func (mr *MockRegistryModulesMockRecorder) List(ctx, organization, options any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "List", reflect.TypeOf((*MockRegistryModules)(nil).List), ctx, organization, options)
}

// ListCommits mocks base method.
func (m *MockRegistryModules) ListCommits(ctx context.Context, moduleID tfe.RegistryModuleID) (*tfe.CommitList, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "ListCommits", ctx, moduleID)
	ret0, _ := ret[0].(*tfe.CommitList)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// ListCommits indicates an expected call of ListCommits.
func (mr *MockRegistryModulesMockRecorder) ListCommits(ctx, moduleID any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "ListCommits", reflect.TypeOf((*MockRegistryModules)(nil).ListCommits), ctx, moduleID)
}

// Read mocks base method.
func (m *MockRegistryModules) Read(ctx context.Context, moduleID tfe.RegistryModuleID) (*tfe.RegistryModule, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Read", ctx, moduleID)
	ret0, _ := ret[0].(*tfe.RegistryModule)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// Read indicates an expected call of Read.
func (mr *MockRegistryModulesMockRecorder) Read(ctx, moduleID any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Read", reflect.TypeOf((*MockRegistryModules)(nil).Read), ctx, moduleID)
}

// ReadTerraformRegistryModule mocks base method.
func (m *MockRegistryModules) ReadTerraformRegistryModule(ctx context.Context, moduleID tfe.RegistryModuleID, version string) (*tfe.TerraformRegistryModule, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "ReadTerraformRegistryModule", ctx, moduleID, version)
	ret0, _ := ret[0].(*tfe.TerraformRegistryModule)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// ReadTerraformRegistryModule indicates an expected call of ReadTerraformRegistryModule.
func (mr *MockRegistryModulesMockRecorder) ReadTerraformRegistryModule(ctx, moduleID, version any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "ReadTerraformRegistryModule", reflect.TypeOf((*MockRegistryModules)(nil).ReadTerraformRegistryModule), ctx, moduleID, version)
}

// ReadVersion mocks base method.
func (m *MockRegistryModules) ReadVersion(ctx context.Context, moduleID tfe.RegistryModuleID, version string) (*tfe.RegistryModuleVersion, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "ReadVersion", ctx, moduleID, version)
	ret0, _ := ret[0].(*tfe.RegistryModuleVersion)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// ReadVersion indicates an expected call of ReadVersion.
func (mr *MockRegistryModulesMockRecorder) ReadVersion(ctx, moduleID, version any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "ReadVersion", reflect.TypeOf((*MockRegistryModules)(nil).ReadVersion), ctx, moduleID, version)
}

// Update mocks base method.
func (m *MockRegistryModules) Update(ctx context.Context, moduleID tfe.RegistryModuleID, options tfe.RegistryModuleUpdateOptions) (*tfe.RegistryModule, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Update", ctx, moduleID, options)
	ret0, _ := ret[0].(*tfe.RegistryModule)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// Update indicates an expected call of Update.
func (mr *MockRegistryModulesMockRecorder) Update(ctx, moduleID, options any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Update", reflect.TypeOf((*MockRegistryModules)(nil).Update), ctx, moduleID, options)
}

// Upload mocks base method.
func (m *MockRegistryModules) Upload(ctx context.Context, rmv tfe.RegistryModuleVersion, path string) error {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Upload", ctx, rmv, path)
	ret0, _ := ret[0].(error)
	return ret0
}

// Upload indicates an expected call of Upload.
func (mr *MockRegistryModulesMockRecorder) Upload(ctx, rmv, path any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Upload", reflect.TypeOf((*MockRegistryModules)(nil).Upload), ctx, rmv, path)
}

// UploadTarGzip mocks base method.
func (m *MockRegistryModules) UploadTarGzip(ctx context.Context, url string, r io.Reader) error {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "UploadTarGzip", ctx, url, r)
	ret0, _ := ret[0].(error)
	return ret0
}

// UploadTarGzip indicates an expected call of UploadTarGzip.
func (mr *MockRegistryModulesMockRecorder) UploadTarGzip(ctx, url, r any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "UploadTarGzip", reflect.TypeOf((*MockRegistryModules)(nil).UploadTarGzip), ctx, url, r)
}
